/**************************************************************
 *
 *  Program Title:  Wasp AVR Monitor
 *  
 *  Program Name:  External_Converters.c	  Version: 0.01
 *  
 *  Author:  BR	  					  Date:  March 26, 2008
 *  
 *  Compiled with ImageCraft ICCAVR Standard V7.02
 *  
 *  This modual provides functions to control the ADS1110 and the DAC8501
 *  These devices are not placed on the WASP by default so they may not be
 *  available to you.  They may however be soldered onto the board if required.
 *
 * Copyright 2008, SOC ROBOTICS, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC ROBOTICS, 
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC ROBOTICS, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SOC ROBOTICS, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-robotics.com
 */


#include <iom644v.h>
#include "External_Converters.h"


unsigned char old_DDR;

/*
 *Error Function
 */
int error(int err)
{
 	return err;
}


/*
 *Sends a control byte to the ADS1110.
 *If the ADC is not connected or if there
 *is an issue interfacing, -1 will be returned
 */
int InitI2CADC(unsigned char control)
{
 	//setting the SCK to ~100k
 	TWBR = 2;
		
	//Send a start condition
 	TWCR = (1<<TWINT)|(1<<TWSTA)|(1<<TWEN);
	//Wait till it's done
	while (!(TWCR & (1<<TWINT)));
	//ensure no error
	if ((TWSR & 0xF8) != 0x08)//start was good
	   return error(-1);
	   
	//send slave address and say we are writing to it
	TWDR = (ADC_ADDRESS<<1);//SLA_W;
	TWCR = (1<<TWINT) | (1<<TWEN);
	
	//Wait till it's done
	while (!(TWCR & (1<<TWINT)));
	
	if ((TWSR & 0xF8) != 0x18)//SLA+W has been transmitted
	   return error(-1);
	
	//send data
	TWDR = control;
	TWCR = (1<<TWINT) | (1<<TWEN);
	//Wait till it's done
	while (!(TWCR & (1<<TWINT)));
	
	if ((TWSR & 0xF8) !=0x28)//Data Ack
	   return error(-1);
	
	//Send stop bit.  No need to wait
	TWCR = (1<<TWINT)|(1<<TWEN)|(1<<TWSTO);
	     
 	return 0;
}


/*
 *This function will read the ADS1110
 *If there is an error, it will return the
 *value of the TWSR register
 */
int ReadI2CADC(void)
{
 	int temp;//will store the read value
	
	//Send a start condition
 	TWCR = (1<<TWINT)|(1<<TWSTA)|(1<<TWEN);
	//Wait till it's done
	while (!(TWCR & (1<<TWINT)));
	//ensure no error
	if ((TWSR & 0xF8) != 0x08)//start was good
	   return error((TWSR & 0xF8));
	   
	//send slave address and say we are writing to it
	TWDR = (ADC_ADDRESS<<1 | 0x01);//SLA_R;
	TWCR = (1<<TWINT) | (1<<TWEN);
	
	//Wait till it's done
	while (!(TWCR & (1<<TWINT)));
	
	
	if ((TWSR & 0xF8) != 0x40)//SLA+R has been transmitted
	   return error((TWSR & 0xF8));
	
	
	//read data
	TWCR = (1<<TWINT) | (1<<TWEN)| (1<<TWEA);//ACK the data
	//Wait till it's done
	while (!(TWCR & (1<<TWINT)));
	
	if ((TWSR & 0xF8) != 0x50)
	   return error((TWSR & 0xF8));
	
	temp = (TWDR<<8);//MSB shifted to the correct location
	
	TWCR = (1<<TWINT) | (1<<TWEN) | (0<<TWEA);//nACK the data
	//Wait till it's done
	while (!(TWCR & (1<<TWINT)));
	
	if ((TWSR & 0xF8) != 0x58)
	   return error((TWSR & 0xF8));
	//Add on the LSB
	temp +=TWDR;
	

	//Send stop bit.  No need to wait
	TWCR = (1<<TWINT)|(1<<TWEN)|(1<<TWSTO);
	     	
	return temp;
}


/*
 *Set up the DAC8501
 */
void InitDAC(void)
{
	 old_DDR = SPI_DDR;//Storing old port directions
	 
	 //set up our port directions.  SS is setup as an output so that the
	 //SPI can not be turned into a slave
	 SPI_DDR |= (1<<MOSI) | (1<<SCLK) | (1<<SS);
	 
	 //Set the enabkle pin as an output
	 DAC_EN_DIR |= (1<<DAC_EN);
	 
	 //Ensure the DAC8501 is not enabled
	 DAC_EN_PORT |= (1<<DAC_EN);
	 
	 //set up the SPI registers
	 SPCR = (1<<SPE) | (1<<MSTR) |(1<<CPOL) | (0<<CPHA);
 	 
}

/*
 *Disable the SPI port and place the port directions
 *back to how they where
 */
void ReleaseDAC(void)
{
	 SPI_DDR = old_DDR;
	 SPCR = 0x00;//Disable the SPI
}

/*
 *Will write the data to the DAC8501
 *
 *            Vref X data
 *   Vout =  ------------- 
 *              65,536
 */
void WriteDAC(unsigned int data)
{
	//select the DAC8501
	DAC_EN_PORT &= ~(1<<DAC_EN);
	
	
	//if you need to modify PD (power-down modes)
	//modify this data
	//ie the data is X, X, X, X, X, X, PD1, PD0
	SPDR = 0x00;
	while(!(SPSR & (1<<SPIF)));
	
	SPDR = data>>8;
	while(!(SPSR & (1<<SPIF)));
	
	
	SPDR = (unsigned char)data;
	
	while(!(SPSR & (1<<SPIF)));
	
	//deselect the DAC8501
	DAC_EN_PORT |= (1<<DAC_EN);

}
