/**************************************************************

   Program Title:  Wasp AVR Monitor
   
   Program Name:  Wasp_Monitor.c	  Version: 0.86
   
   Author:  SS, BR	  			      Date:  March 26, 2008
   
   Compiled with ImageCraft ICCAVR Standard V7.02
   
   This verison is designed for the ATmega644 processor:
   		64K Flash, 2K EEPROM, 4K RAM
		
   Fuse bit:  FF D9 E2  - Disable JTAG - use internal OSC
	
   This program will not run properly if DIV8 fuse bit is set (programmed)
   and JTAG pins are enabled - this is the chip default.
		
   The Wasp Monitor is a set of functions that configure Wasp AVR internal
   peripherals for interrupt driven operation.  Custom applications can easily
   be built on top to these basic functions.  Included is a complete data 
   acquistion controller with user preferences loaded into EEPROM.  The Monitor
   communicates with the user via an RS-232 port using the CDP.   The AVR is 
   a smart powerful 8 bit RISC processor that can be easily programmed in C
   with on chip TWI, SPI and analog and digital IO.  Functions are included 
   to read/write data blocks to the on board Serial Flash.  On board sensors
   (if included) include a dual axis 2G accelerometer, temperature and light
   sensor and battery voltage monitor.
   
   Sensors:
   		AD0 - Accelerometer channel X
		AD1 - Accelerometer channel Y
		AD2 - Battery Voltage
		AD3 - Temperature 
		AD4 - Light level
		
   Key functions:
   	   SPI Master with read/write function to serial flash
	   TWI 2-Wire Master Interrupt Controller
	   Real Time Clock setup with four configurable software timers
	   Full duplex interrupt drive USART with XON/XOFF flow control
	   1msec timer with four configurable software times
	   Analog controller setup
	   Digital port configuration
	   Accelerometer reader
	   Temperature reader
	   Light level reader
	   IntelHex32 download/upload for EEPROM, AVRFLASH and Serial Flash
	   Auto clock calibration
	   
   Standard printf and scanf functions are not used in this version 
   due to code size constraints.  The Wasp is available with an ATmega32
   with twice the Flash and SRAM if printf and scanf functionality is 
   important or a custom application requires more than the available
   AVR Flash.
   
   Copyright 2004-2008, SOC ROBOTICS, Inc.  All rights reserved.
   
****************************************************************/
/*
 * Copyright 2004-2008, SOC ROBOTICS, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC ROBOTICS, 
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC ROBOTICS, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SOC ROBOTICS, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-robotics.com
 */

/*
 * $Log: Wasp_Monitor.c,v $
 * Revision 0.86  2006/02/07 16:40:20  SS
 *    2-wire interface, 8MHz clock, Serial Flash writer, XON/XOFF, 9,600baud
 *    real time clock and auto-internal osc calibration.
 * Revision 0.52  2005/06/04 18:36:42  SS
 * Preview release
 * 
 */ 
#include <iom644v.h>
#include <macros.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include "Wasp_Uart.h"
#include "Wasp_Defines.h"
#include "Wasp_Extern.h"

#include "External_Converters.h"

#define   G15MODE   '1'
#define   G2MODE    '2'
#define   G4MODE    '4'
#define   G6MODE    '6'


//  Variable declarations
unsigned int bytesonline,inteladdress;
unsigned char programmode,crc,intelbinary[34];  // length can vary
unsigned char hexbyte[3],hexaddress[5],intelrecordcommand;
unsigned char seconds,minutes,hours,days,rtcclockcnt;
unsigned char flashtype0,flashtype1,flashpageloaded,flashpageboundarytraversed;
unsigned char flashprogrammingactive,flashdatainbuffer,ascii[16];
unsigned int flashpagesize0,flashpagesize1,oldinteladdress;
unsigned int intelsuperaddress,oldpage,oldbyteoffset,totalrxcnt;
unsigned int rtccountdown1,rtccountdown2,rtccountdown3,rtccountdown4;
unsigned int mseccountdown1,mseccountdown2,mseccountdown3,flashcountdown;

unsigned int totalbytecnt,xoncnt,xoffcnt;
unsigned char frameerror,dataoverrun;
int  cursorlocation;

extern int _textmode;

/* Static Variables */
unsigned char UART_RxBuf[UART_RX_BUFFER_SIZE];
unsigned char UART_RxHead;
unsigned char UART_RxTail;
unsigned char UART_TxBuf[UART_TX_BUFFER_SIZE];
unsigned char UART_TxHead;
unsigned char UART_TxTail;
char  xoffsent, xonsent, enablexonoff, echoinchars,sendxoff;
char  rxcnt,checkflag,rxcntmax;
unsigned char  instrg[8],vlow,vhigh;
float sensitivity;

unsigned char FlashAddress[3];
const char signon1[] = "\n\r\n\rWasp Monitor V0.85\n\r";
const char signon2[] = "Copyright 2004-2006,  SOC Robotics, Inc.\n\r\n\r";
const char signon3[] = "   D[E|A|F]  - Display AVR EEPROM, AVR Flash or Flash memory\n\r";
const char signon4[] = "   P[E|A|F]  - Program AVR EERPOM, AVR Flash or Flash memory\n\r";
const char signon5[] = "   F   - Measure force in lbs\n\r";
const char signon6[] = "   G   - Measure gravity using accelerometer\n\r";
const char signon7[] = "   V   - Version\n\r";
const char signon8[] = "   C   - Test AVR\n\r";
const char signon83[] = "   T   - Get/Set Time\n\r";
const char signon85[] = "   A   - Read all Analog channels\n\r";
const char signon9[] = "   S   - Signon message\n\r\n\r-";

const char displaymessage1[] = "Enter start address and length: ";
const char displaymessage2[] = "\n\rDisplay AVR Flash from 0x0054 to 254\n\r";
const char displaymessage3[] = "\n\rDisplay SPI Flash from 0 to 100\n\r";
const char displaymessage33[] = "\n\rNo mode selected\n\r";

const char progmessage[] = "Wasp Monitor Version: ";

const char displaymessage4[] = "\n\rLoad EEPROM from address ";
const char displaymessage5[] = " length ";
const char displaymessage6[] = "\n\rLoad SPI Flash from address ";

const char loadmessage1[] = "\n\rLoad EEPROM\n\r";
const char loadmessage2[] = "\n\rLoad SPI Flash\n\r";
const char loadmessage3[] = "\n\rLoad AVR Flash\n\r";

// Function declarations
unsigned char EEPROMwrite(int,unsigned char);
unsigned char EEPROMread(int);
int printf(CONST char *, ...);
int sprintf(char *, CONST char *, ...);
int scanf(CONST char *, ...);
int sscanf(char *, CONST char *, ...);
void Display_External_ADC();

void Draw_Boxes_Once()
{
 	unsigned char inchar;
	
	Draw_Background(0xff);
	printf("Draw boxes\n");
	Draw_Color_Bars(28);
	Draw_Boxes(38);
	LongDelay(40);
	Draw_Background(0xff);
}

void Draw_Boxes_Continuously()
{
 	unsigned char inchar;
	
	printf("3\nDraw boxes continuously");
	printf(" - press any key to exit\n");
	Draw_Background(0xff);
	while(!(inchar=getcharnw())) {
		printf("Draw boxes\n");
		Draw_Color_Bars(28);
		Draw_Boxes(38);
		LongDelay(40);
		Draw_Background(0xff);
		}
}

void Draw_Menu_Command()
{
	printf("m\nEnter Menu command:");
	Draw_Background(0xff);
	cursorlocation = 0;
	//Draw_Menu();
	LCD_Puts(9,113,"LED Off/On - Right SW",0x03);
	LCD_Puts(9,99,"Boxes - Sw right",0x03);
	LCD_Puts(9,85,"Alphabet - Up",0x03);
	LCD_Puts(9,72,"Force - Down",0x03);
	LCD_Puts(9,59,"Gravity - Left",0x03);
	LCD_Puts(9,46,"SOC Logo - Down",0x03);
	LCD_Puts(9,33,"Erase Scren - Center",0x03);
							
	LCD_Puts(1,113,">",0x03);
	printf("\n-");
}


// Initial MMA7260 3-Axis accelerometer
void Init_3Axis_Accelerometer(char mode)
{
 	 // Set direction bits
	 //   SG2 - PC5, SG1 - PC4, SLP - PC3
	 //DDRC |= 0x3C;
	 DDRC |= 0x38;
	 
	 
 	 // Enable Accelerometer
	 SetBit(PORTC,PORTC3);
	 //ClrBit(PORTC,PORTC3);
	 
	 if(mode==G15MODE) {
	 	ClrBit(PORTC,PORTC5);
		ClrBit(PORTC,PORTC4);
		sensitivity = 0.8/3.3; //800mV/g -> The sensitivity is 0.8mV/g on a 3.3
					  		   //supply.  This is ratiometric so we can use
							   //it as a fraction
		
	 } else if(mode==G2MODE) { 
	    ClrBit(PORTC,PORTC5); 
		SetBit(PORTC,PORTC4);
		sensitivity = 0.6/3.3; //600mV/g 
	 } else if(mode==G4MODE) { 
	    SetBit(PORTC,PORTC5); 
		ClrBit(PORTC,PORTC4);
		sensitivity = 0.3/3.3; //300mV/g 
	 } else if(mode==G6MODE) { 
	    SetBit(PORTC,PORTC5); 
		SetBit(PORTC,PORTC4);
		sensitivity = 0.2/3.3; //200mV/g 
	 }
}

void Display_Gravity()
{
    unsigned char firstmeasurement;
	char outstrg[30],inchar;
    int i,xgrav,ygrav,zgrav,xmin,xmax;
	long xgravity,ygravity,zgravity;
	float gravity,xaccel,yaccel,force,bforce,cforce,dforce,fforce,roll,pitch;
	float xfgrav,yfgrav,zfgrav;
	
	Init_3Axis_Accelerometer(G15MODE);
	firstmeasurement = YES;
	Draw_Background(0xff);
	LCD_Puts(1,114,"SOC Robotics",0x00);
	LCD_Puts(4,90,"Acceleration",0x00);
	LCD_Puts(4,74,"x axis = ",0x00);
	LCD_Puts(4,60,"y axis = ",0x00);
	LCD_Puts(4,46,"z axis = ",0x00);
	LCD_Puts(4,26,"Press right to",0x00);
	LCD_Puts(4,12,"exit",0x00);

	while(1) {
		xgravity = ygravity = zgravity = 0;
		for(i=0; i<10; i++) {
			xgravity += AnalogCh(5,EXTERNALREF);
			ygravity += AnalogCh(0,EXTERNALREF);
			zgravity += AnalogCh(1,EXTERNALREF);
			}
			
		xgravity = xgravity/10;
		ygravity = ygravity/10;
		zgravity = zgravity/10;
		
		//xfgrav = ((float)(512.0 - xgravity))/250.0;
		//yfgrav = ((float)(512.0 - ygravity))/250.0;
		//zfgrav = ((float)(512.0 - zgravity))/250.0;
		
		
		/*
		Here is how we work out the acceleration
	           (ADC/1024) - 0.5
	     g =   -----------------
		              s
	    
		Please note that this funtion does not calibrate.
	    It assumes that 0g is at the mid point of the voltage rails.
		However 0g on a 3.3v supply can vary from 1.485 to 1.815 typ 1.65
		
		
		*/
		
		
		xfgrav = ((xgravity/1024.0)-0.5)/sensitivity;
		yfgrav = ((ygravity/1024.0)-0.5)/sensitivity;
		zfgrav = ((zgravity/1024.0)-0.5)/sensitivity;
		
		sprintf(outstrg,"%#6.3f",xfgrav);
		LCD_Puts(66,74,outstrg,0x00);
		sprintf(outstrg,"%#6.3f",yfgrav);
		LCD_Puts(66,60,outstrg,0x00);
		sprintf(outstrg,"%#6.3f",zfgrav);
		LCD_Puts(66,46,outstrg,0x00);
		inchar = Read_Switches();
		if(inchar==SWITCH_DOWN) break;
		if(inchar==SWITCH_UP) break;
		}
}

void Display_Gravity2()
{
    unsigned char firstmeasurement;
	char outstrg[30];
    int i,xgrav,ygrav,zgrav,xmin,xmax;
	long xgravity,ygravity,zgravity;
	float gravity,xaccel,yaccel,force,bforce,cforce,dforce,fforce,roll,pitch;
	float xfgrav,yfgrav,zfgrav;
	
	Init_3Axis_Accelerometer(G15MODE);
	
	//Init_3Axis_Accelerometer(G4MODE);
	
	Draw_Background(0xff);
	LCD_Puts(1,114,"SOC Robotics",0x00);
	LCD_Puts(4,90,"Acceleration",0x00);
	LCD_Puts(4,74,"x axis = ",0x00);
	LCD_Puts(4,60,"y axis = ",0x00);
	LCD_Puts(4,46,"z axis = ",0x00);
	LCD_Puts(4,26,"Press right to",0x00);
	LCD_Puts(4,12,"exit",0x00);

	while(1) {
		xgravity = ygravity = zgravity = 0;	
		for(i=0; i<1; i++) {
			xgravity += AnalogCh(5,EXTERNALREF);
			ygravity += AnalogCh(0,EXTERNALREF);
			zgravity += AnalogCh(1,EXTERNALREF);
			}

		xgravity = (xgravity/1);
		ygravity = (ygravity/1);
		zgravity = (zgravity/1);

		sprintf(outstrg,"%4d   ",xgravity);
		LCD_Puts(70,74,outstrg,0x00);
		sprintf(outstrg,"%4d   ",ygravity);
		LCD_Puts(70,60,outstrg,0x00);
		sprintf(outstrg,"%4d   ",zgravity);
		LCD_Puts(70,46,outstrg,0x00);
		}
}

/*
 *A simple function that will test the DAC8501 (U5)
 *Note that this device is not include on the board
 *by default.
 *It will output a square wave
 */
void Cycle_DAC()
{
	 InitDAC();
	 shortdelay(20);
	
	 while (1)
	 {
	  	   WriteDAC(0);
		   WriteDAC(0xffff);
	 }
}

/*
 *This function will display the reading on a LCD
 *From at 16 ADC  (ADS1110)
 *Note that this device is not include on the board
 *by default.
 */
void Display_External_ADC()
{
 	char outstrg[30];

	
	Draw_Background(0xff);
	LCD_Puts(1,114,"SOC Robotics",0x00);
	LCD_Puts(4,90,"External ADC",0x00);
	LCD_Puts(4,74,"ADC = ",0x00);
	
	if(InitI2CADC((0<<SC)|(3<<DR))){//continuous conversion 15 samples per second
								   
		LCD_Puts(70,60,"Error",0x00);
	}

	while(1) {
		sprintf(outstrg,"%4d   ",ReadI2CADC());
		LCD_Puts(70,74,outstrg,0x00);
		}
}



//*************************************************************
//  Main work starts here
//*************************************************************
void main()
{
	unsigned char inchar,inbyte,twistatus1,twistatus2,twistatus3,twistatus4;
	unsigned char inbytes[5],toggle,slaverx,firstmeasurement;
	char outstrg[30],LCD_Switches;
	int i,icnt,j,loopsetting,loopmode,blackfinresetmode,countervalue,numbr;
	int loopmode2,num1,num2,xgrav,ygrav,xmin,xmax,iforce,dataoutputperiod;
	int ylocation,centerpressed;
	long xgravity,ygravity,loopcount,ivalue,bvalue,cvalue;
	float gravity,xaccel,yaccel,force,bforce,cforce,dforce,fforce,roll,pitch;
	
	_textmode = 1;  // Expand /n to CRLF
	Init_AVR();
	rxcntmax = totalrxcnt = 0;
	
	loopsetting = 100;   // 1 second delay
	loopmode = 0;  toggle = 0;
	mseccountdown1 = loopsetting;
	loopcount = 0;
	dataoutputperiod = 10;  // seconds
	LCD_Switches = 0x00;   // Switch toggle values

	// Get Serial Flash status
//	CheckFlashTypes();
	
	// Show something is happening
	while(0) {  
		BlinkRed(2);
		BlinkGreen(2);
		Nokia_LCD_LED(ON);
		Delaymsec(500);
		Nokia_LCD_LED(OFF);
		}
		
	BlinkRed(2);
	BlinkGreen(2);
	Nokia_LCD_LED(ON);
	Delaymsec(500);
	Nokia_LCD_LED(OFF);
	
	signon();
	
	Nokia_LCD_LED(ON);
	Initialize_Nokia_LCD();
	
	
	GREEN_LED(ON);
							
	
//	WriteEEPROM(0,"SOC Robotics V0.85");
	Draw_Menu_Command();
	// Wait for commands on serial channel
	while (1) {
		  // Work starts here
//		  if((inchar = getcharnw())) {
		  if(0) {
			   	switch (inchar) {
					 case 's': case 'S':
						 signon();
						 //printf("rxcnt=%d\n-",rxcnt); 
						 break;
					    case 'd': case 'D':
						    putcharr(inchar);
						    DisplayMemory();
							putcharr('-');
							break;
					    case 'p': case 'P':
							putcharr(inchar);
						    LoadMemory();
							putcharr('-');
							break;
					    case 'a': case 'A':
						    DumpAnalogChs();
							xgravity = ygravity = 0;
							for(i=0; i<10; i++) {
								xgravity += AnalogCh(0,INTERNALREF);
								ygravity += AnalogCh(1,INTERNALREF);
								}
							xgrav = xgravity/10;
							ygrav = ygravity/10;
							printf(" Accelerometer x-axis=%3d\n",xgrav);
							printf(" Accelerometer y-axis=%3d\n",ygrav);
							// Reference battery voltage to internal 2.56v ref otherwise
							// input voltage value will not change as battery voltage drops
							printf(" Battery voltage=%3d\n",AnalogCh(2,INTERNALREF));
							printf(" Temperature sensor=%3d\n",AnalogCh(3,EXTERNALREF));
							printf(" Light sensor=%3d\n",AnalogCh(4,EXTERNALREF));
							putcharr('-');
							break;
					    case 'v': case 'V':
						    ProgramVersion();
							putcharr('-');
							break;
					    case 'f': case 'F':
						    putcharr('f');
							printf("\nForce sensor output:\n");
							Draw_Background(0xff);
							LCD_Puts(1,114,"SOC Robotics",0x03);
							LCD_Puts(4,90,"Force =  ",0x03);
							while(!(inchar=getcharnw())) {
				   	 		    iforce = AnalogCh(7,EXTERNALREF);
								//force = (1024.0 - iforce);
								//if(iforce>840) force = 1.0 + 0.0125*(185.0-force);
								//else force = 3.3*(1/force);
								force = iforce * 0.00414;
								fforce = iforce*1.0;
								bforce = 3.83e-5*(fforce*fforce);
								cforce = -8.9522e-8*(fforce*fforce*fforce);
								dforce = 5.9e-11*(fforce*fforce*fforce*fforce);
								force = 0.31+bforce+cforce+dforce;
								if(iforce<2) force = 0.0;
								printf(" f=%3d %#7.3f lbs      \r",iforce,force);
//								printf(" f=%d %.3f lbs  %f %f %f %f    \r",
//								      iforce,force,fforce,bforce,cforce,dforce);
								sprintf(outstrg,"%#7.3f",force);
								LCD_Puts(55,90,outstrg,0x03);
							    }
							printf("\n-");
							break;
					    case ':':
						    ProgramMemory();
							//putcharr('-');
							break;
						case 'l': case 'L':
						    while(!(inchar=getcharnw())) putcharr(0x55);
							printf("\n-");
							break;
						case 'c': case 'C':
						    while(!(inchar=getcharnw()))  TestAVR();
							signon();
							break;
						case 't': case 'T':
						    getsettime();
							printf("\n-");
							break;
						case 'x': case 'X':
							putcharr(inchar);
							TWI_Write_Slave();
							putcharr('-');
							break;
						case 'q': case 'Q':
							putcharr(inchar);  //puts("\n\rTWI Master Read Data ");
						    // while(!(inchar=getcharnw()))  TestTWI();
							Init_TWI_Master();
							inbyte = TWI_Master_Receive(0x15);
						    //while(!(inchar=getcharnw()))  {
							    // Send start command and wait for completion
								/*TWCR = (1<<TWINT)|(1<<TWEN)|(1<<TWSTA);
								while(!(TWCR & (1<<TWINT)))  ;	
								twistatus1 = TWSR;
								// Send slave address and read command 
								TWDR = (0x15<<1) | SLA_R;
								TWCR = (1<<TWINT)|(1<<TWEN);
								while(!(TWCR & (1<<TWINT)))  ;  
								twistatus2 = TWSR; 
								// Read data byte and wait for transmission to complete
	  							TWCR = (1<<TWINT)|(1<<TWEA)|(1<<TWEN);
								countervalue = 0;
	  							while(!(TWCR & (1<<TWINT))) {
									if(countervalue++>40) break;
									} 	
								inbyte = TWDR;    
								twistatus3 = TWSR;
								// Read next data byte and wait for transmission to complete
	  							TWCR = (1<<TWINT)|(1<<TWEN);
								countervalue = 0;
	  							while(!(TWCR & (1<<TWINT))) {
									if(countervalue++>40) break;
									} 	
								//inbyte = TWDR;    
								twistatus4 = TWSR;
								
								if(countervalue>40) {
								   TWI_Off();
								   Init_TWI_Master();
								   }
								else TWCR = (1<<TWINT)|(1<<TWEN)|(1<<TWSTO);
								shortdelay(5);  
								//}
							    TWI_Off();
							    printf("\n\rTWI status bytes = %x %x %x %x"
								  twistatus1,twistatus2,twistatus3,twistatus4);
								*/
								TWI_Off();
							    //printf("\n\rTWI Master Read byte = %x  %c\n\r-",inbyte,inbyte); 
							    break;
							case 'y': case 'Y':
						        // This version assumes single byte reception
							    putcharr(inchar);  puts(" TWI Slave Active ");
						        // while(!(inchar=getcharnw()))  TestTWI();
							    slaverx = 1;  icnt = 0;
							    Init_TWI_Slave(0x15,TWI_GENERAL_CALL_ON);
						        while(!(inchar=getcharnw()))  {
							        // If slave address match process data
								    // otherwise loop and exit if keyboard key pressed
								    if((TWCR & (1<<TWINT))) {
								        for(i=0; i<5; i++)  inbytes[i] = 0;
									    i = 0;	slaverx = 2;
								        // Clear flag and wait for data reception
									
									    // If data receive command - read until
									    //   stop condition received
									    if((TWSR&0xF8)==SR_WRITE_COMMAND) {
								            TWCR = (1<<TWINT)|(1<<TWEA)|(1<<TWEN);
								            while(!(TWCR & (1<<TWINT)))  ;
									
									        // Read data (up to 5 bytes) 
									        //   until stop condition detected
									        //   TWEA send ACK
								            while((TWSR&0xF8)!=TWI_STOP)  {
										        if(i>=5) break;
									            inbytes[i++] = TWDR; 
										        icnt = i;     			 
	  							                TWCR = (1<<TWINT)|(1<<TWEA)|(1<<TWEN);
	  							                while(!(TWCR & (1<<TWINT)))  ; 	  
								                TWCR = (1<<TWINT)|(1<<TWEA)|(1<<TWEN);
								                } 
										    }
									    // If data send command - send data with
									    //   TWEA set until last byte 
									    //   this example sends only one byte 
									    else if((TWSR&0xF8)==SR_READ_COMMAND) {
									    //else  {
									        putcharr('v');
									        TWDR = 0x78;  // data to send
										    TWCR = (1<<TWINT)|(1<<TWEN);
										    //TWCR = (1<<TWINT)|(1<<TWEN);
										    while(!(TWCR & (1<<TWINT)))  ;
										    TWCR = (1<<TWINT)|(1<<TWEA)|(1<<TWEN); 
										    }
	 								    }
								    }
							TWI_Off();
							if(slaverx==2) {
								printf("\n%d",icnt); 
								//puts(" Slave data bytes rx'd = "); 
							    for(i=0; i<icnt; i++) {
								    bytetohex(inbytes[i]);
								    puts(hexbyte); putcharr(' ');
									}
								for(i=0; i<icnt; i++) {
								    putcharr(inbytes[i]);
									}
								}
							else  ;//puts("\n\rNo Slave data rx'd");
							//shortdelay(6);
							//TWCR = (1<<TWINT)|(1<<TWEN);
							puts("\n\r-");
							break;
					    case '1':
						    printf("4\nTurn LCD contrast up");
						    Nokia_LCD_ContrastUp();
							printf("\n-");
							break;
					    case '2':
						    printf("5\nTurn LCD contrast down");
						    Nokia_LCD_ContrastDown();
							printf("\n-");
							break;
					    case '6':
						    printf("6\nTurn LCD LED on");
						    Nokia_LCD_LED(ON);
							printf("\n-");
							break;
					    case '7':
						    printf("7\nTurn LCD LED off");
						    Nokia_LCD_LED(OFF);
							printf("\n-");
							break;
						case 'n':
						    putcharr('n');
							printf("\nInitialize Nokia LCD display\n");
							Initialize_Nokia_LCD();
							printf("\n-");
							break;							
						case '8':
						    putcharr('8');
							printf("\nSwitch values=");
							printf("%x",Read_Switches());
							printf("\n-");
							break;							
						case '9':
						    putcharr('9');
							printf("\nDisplay SOC Logo\n");
							ylocation = 0;
							while(!(inchar=getcharnw())) {
							    Draw_SOC_Logo(ylocation+1,0);
							    Draw_Box_Filled(ylocation,0,ylocation,128,28);
								ylocation++;
								if(ylocation==40) {
								    ylocation = 0;
									Draw_Box_Filled(40,0,110,128,28);
									}
								}
							printf("\n-");
							break;							
						case '4':
						    printf("4\nOutput numbers continuously");
							printf(" - press any key to exit\n");
							ivalue = 0;
							bvalue = 34;
							cvalue = 34;
							gravity = 0.0;
							bforce = 23.12;
							cforce = 34.23;
							dforce = 54.34;
							Draw_Background(0xff);
							while(!(inchar=getcharnw())) {
								sprintf(outstrg,"%#6.2f",dforce);
								dforce -= 0.03;
								LCD_Puts(50,99,outstrg,0x03);
								sprintf(outstrg,"%6d",cvalue++);
							    LCD_Puts(50,85,outstrg,0x03);
								sprintf(outstrg,"%#6.2f",cforce);
								cforce += 0.54;
								LCD_Puts(50,72,outstrg,0x03);
								sprintf(outstrg,"%#6.2f",bforce);
								bforce += 0.34;
								LCD_Puts(50,59,outstrg,0x03);
								sprintf(outstrg,"%6d",bvalue++);
							    LCD_Puts(50,46,outstrg,0x03);
								sprintf(outstrg,"%6d",ivalue++);
							    LCD_Puts(50,33,outstrg,0x03);
								sprintf(outstrg,"%#6.2f",gravity);
								gravity += 0.12;
								LCD_Puts(50,20,outstrg,0x03);
								}
							printf("\n-");
							break;
						case 'm':
							Draw_Menu_Command();
							break;
						case '5':
						    printf("4\nDraw alphabet");
							Draw_Background(0xff);
							Draw_Alphabet();
							printf("\n-");
							break;
						case '3':
						    Draw_Boxes_Continuously();
							printf("\n-");
							break;
						case 'u':
							loopsetting += 100;
							break;
						case 'g':
						    putcharr('g');
							printf("\nOutput gravity continuously\n");
							firstmeasurement = YES;
							Draw_Background(0xff);
							LCD_Puts(1,114,"SOC Robotics",0x03);
							LCD_Puts(4,90,"Roll =  ",0x03);
							LCD_Puts(4,74,"Pitch = ",0x03);
							while(!(inchar=getcharnw())) {
				   	 		    xgravity = ygravity = 0;
								mseccountdown3 = 1000;
								for(i=0; i<10; i++) {
								   xgravity += AnalogCh(0,INTERNALREF);
								   ygravity += AnalogCh(1,INTERNALREF);
								   }
							    xgrav = xgravity/10;
							    ygrav = ygravity/10;
								if(firstmeasurement==YES) {
								    xmin = xmax = xgrav;
									firstmeasurement = NO;
									}
							    if(xgrav<xmin)  xmin = xgrav;
							    if(xgrav>xmax)  xmax = xgrav;
								xaccel = xgrav-515;
								xaccel /= 60.0;
								//for(i=0; i<10; i++) {
								//	yaccel = sin(.45);
								//	yaccel = cos(.45);
								//	}
								yaccel = ygrav-511;
								yaccel /= 62.0;
								roll = asin(xaccel)*57.2957;
								pitch = asin(yaccel)*57.2957;
//							    printf(" g=%3d %3d   %3d %3d  %.3f %.3f    \r",
//								          xgrav,ygrav,xmin,xmax,xaccel,yaccel);
							    //printf(" g=%#7.3f %#7.3f  r/p=%#7.3f %#7.3f  \r",
								//          xaccel,yaccel,roll,pitch);
								sprintf(outstrg,"%#6.2f",roll);
								LCD_Puts(70,90,outstrg,0x03);
								sprintf(outstrg,"%#6.2f",pitch);
							    LCD_Puts(70,74,outstrg,0x03);
								//printf(" g=%x %x %d   \r",vlow,vhigh,xgrav);
							    }
							printf("\n-");
							break;
						case 'e':
						    putcharr('e');
						    Init_SPI_Master();
							inteladdress = 0x0000;
							EraseFlashPage();
							WaitForFlashFree(1);
							inteladdress = 0x0109;
							EraseFlashPage();
							WaitForFlashFree(1);
							inteladdress = 0x0211;
							EraseFlashPage();
							WaitForFlashFree(1);    
							inteladdress = 0x03A0;
							EraseFlashPage();
							WaitForFlashFree(1);    
							Release_SPI();
							printf("\n-");
							break;
						case '=':
							printf("rxcntmax=%d\n-",rxcntmax);
							rxcntmax = 0;
							printf("totalrxcnt=%d\n-",totalrxcnt);
							rxcntmax = 0;
							//if(frameerror==YES) puts("frame error\n\r");
							//else puts("no frame error\n\r");
							//if(dataoverrun==YES) puts("data overrun\n\r");
							//else puts("no data overrun\n\r");
							//dataoverrun = NO;
							//frameerror = NO;
							printf("xoncnt=%d\n",xoncnt);
							xoncnt = 0;
							xoffcnt = 0;
							break;
						case CR:
							if(flashprogrammingactive==NO)  printf("\n-");
							break;
						default:
							putcharr(inchar);
							//Delaymsec(20);
							break;
						}
			   		}
			   
			 //Display_Gravity();
			// Cycle_DAC();
			 Display_External_ADC();
			 //Draw_Menu_Command();
			 //Draw_Background(0xff);
			 //Draw_Alphabet();
			 //Draw_Boxes_Once();
			 //Draw_Background(0x00);
			 //Draw_SOC_Logo(ylocation+1,0);
			 //Cycle_LCD();
			 
			 if(mseccountdown2==0) {
			 		//inchar = Read_Switches();
			 		//sprintf(outstrg,"%d",inchar);
					//LCD_Puts(4,20,"Switch = ",0x00);
					//LCD_Puts(70,20,outstrg);
			      	switch(Read_Switches()) {
			            case SWITCH_RIGHT: 
					        if(LCD_Switches&SWITCH_RIGHT)  {
						        Nokia_LCD_LED(ON);
							    LCD_Switches &= ~SWITCH_RIGHT;
							    }
						    else {
						        Nokia_LCD_LED(OFF);
							    LCD_Switches |= SWITCH_RIGHT;
							    }
							mseccountdown2 = 440;
						    break;
						case SWITCH_UP: 
							if(centerpressed==YES) {
							    Draw_Menu_Command();
								centerpressed = NO;
								}
						    LCD_Puts(1,113-cursorlocation*13," ",0x03);
							if(cursorlocation--==0) cursorlocation = 6;
						    LCD_Puts(1,113-cursorlocation*13,">",0x03);
							mseccountdown2 = 190;
							
							
							RED_LED(ON);
							LongDelay(30);
							RED_LED(OFF);
							break;
						case SWITCH_DOWN: 
							if(centerpressed==YES) {
							    Draw_Menu_Command();
								centerpressed = NO;
								}
						    LCD_Puts(1,113-cursorlocation*13," ",0x03);
							if(cursorlocation++==6) cursorlocation = 0;
						    LCD_Puts(1,113-cursorlocation*13,">",0x03);
							mseccountdown2 = 190;
							RED_LED(ON);
							LongDelay(30);
							RED_LED(OFF);
							
							break;
						case SWITCH_CENTER: 
						    switch(cursorlocation) {
								case 0:
								    break;
								case 1:
						    		centerpressed = YES;
								    Draw_Background(0xff);
									Draw_Color_Bars(28);
									Draw_Boxes(38);
								    break;
								case 2:
									centerpressed = YES;
									Draw_Background(0xff);
									Draw_Alphabet();
								    break;
								case 3:
								    break;
								case 4:
									centerpressed = YES;
									Display_Gravity();
								    break;
								case 5:
								    centerpressed = YES;
									//Draw_Background(0xff);
									Draw_SOC_Logo(ylocation+1,0);
								    break;
								case 6:
								    centerpressed = YES;
									Draw_Background(0xff);
								    break;
							    }
							break;
						case SWITCH_MIDRIGHT: 
						    Draw_Box_Filled(110,10,120,20,03);
							break;
						case SWITCH_MIDLEFT: 
						    Draw_Box_Filled(110,10,120,20,0xFF);
							break;
					    }
			   		} 
								
			   // Blink green led slowly to show AVR is alive
			   if(mseccountdown1==0) {
			   		mseccountdown1 = loopsetting;
					if(loopmode)  {
					    GREEN_LED(ON);
						loopmode = 0;
						}
					else {
						GREEN_LED(OFF);
						loopmode = 1;
						}
			   	 	}
				
								
				// Clean up flash state machine if no new Intel
				// Hex records arrive for 3 seconds
				if((flashcountdown==0)&&(flashprogrammingactive==YES))
				        CleanUpFlashState();
		}
}
